from Products.CMFDefault.SkinnedFolder import SkinnedFolder
from Globals import InitializeClass
from DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from AccessControl import ClassSecurityInfo
from Acquisition import aq_base

from md5 import md5
from rssparser import parse
from RSSStory import RSSStory
from binascii import b2a_base64
from RSSFeed import RSSFeed, STATUS_DEAD, STATUS_OK
from Permissions import p_manageRSSCollection, p_View

import string

#For debugging
from logger import logger

_replace_table = [
    ('_', '/:%$ \t'),
    ('-', '=+!'),
    (',', '.&@?*<>#;'),
    ]
_delete_chars = '\'"'
_replace_chars = ''.join([ x[1] for x in _replace_table ])
_replace_by = ''.join([ (x[0] * len(x[1])) for x in _replace_table ])
_base64_to_id_trans = string.maketrans( _replace_chars, _replace_by )

factory_type_information = ({'id': 'RSSCollection',
                             'content_icon': 'folder_icon.gif',
                             'meta_type': 'RSSCollection',
                             'description':""" """,
                             'product': 'CMFAggregator',
                             'factory': 'addRSSCollection',
                             'immediate_view': 'rsscollection_edit_form',
                             'allowed_content_types':['RSSFeed', 'RSSCollection'],
                             'actions':({'id': 'view',
                                          'name': 'View',
                                          'action': 'rsscollection_view',
                                          'permissions': (p_View,)},
                                         {'id': 'contents',
                                          'name': 'Contents',
                                          'action': 'folder_contents',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'edit',
                                          'name': 'Edit',
                                          'action': 'portal_form/rsscollection_edit_form',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'aggregate',
                                          'name': 'Aggregate',
                                          'action': 'aggregate',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'import',
                                          'name': 'Import',
                                          'action': 'rsscollection_import_form',
                                          'permissions': (p_manageRSSCollection,)},
                                         {'id': 'schedule',
                                          'name': 'Schedule',
                                          'action': 'rsscollection_schedule_form',
                                          'permissions': (p_manageRSSCollection,)},
                                        ),
                             },
                            )

def addRSSCollection( self, id, title='', description=''):
    """ """
    o=RSSCollection( id=id, 
                     title=title, 
                     description=description
                    )
    self._setObject(id,o)

class RSSCollection(SkinnedFolder):
    """ """
    meta_type = 'RSSCollection'

    security = ClassSecurityInfo()

    def __init__(self, id, title='', description=''):
        """ """
        self.id = id
        self.title = title
        self.description = description

    def edit(self,id='', title='', description=''):
        """ """
        if id:
            self.id = id
        self.title = title
        self.description = description

    security.declarePrivate('syncStory')
    def syncStory(self, feedobj, id, fields):
        """ """
        if feedobj.last_updated < DateTime():
            feedobj.editLastUpdated(DateTime())

    security.declarePrivate('newStory')
    def newStory(self, feedobj, id, fields):
        """ """
        (title, description, link) = fields
        feedobj.invokeFactory(type_name='RSSStory',
                              id=id, 
                              title=title, 
                              description=description,
                              link=link
                              )
        if feedobj.last_updated < DateTime():
            feedobj.editLastUpdated(DateTime())

    security.declarePrivate('getStoryId')
    def getStoryId(self, fields):
        """ """
        id = b2a_base64(md5(''.join(fields)).digest()).strip()
        id = id.translate(_base64_to_id_trans, _delete_chars)
        while id and (id[0] == '_' or id[0] == '-'):
            id=id[1:]
        while id and (id[-1] == '_' or id[-1] == '-'):
            id=id[:-1]
        if not id:
            id = 'error_id'
        return id

    security.declarePrivate('parseStory')
    def parseStory(self,feedobj, story):
        """ """
        fields = story.get('title', 'unknown'), story.get('description',''), story.get('link','')
        id = self.getStoryId(fields=fields)
        if hasattr(feedobj.aq_explicit, id):
            self.syncStory(feedobj, id, fields=fields)
        else:
            self.newStory(feedobj, id, fields=fields)

    security.declarePrivate('parseFeed')
    def parseFeed(self, feedobj):
        """ parse feed object to get stories """
        dataurl = feedobj.dataurl
        try:
            logger(0, 'Try to parse url: %s' %dataurl)
            results= parse(dataurl)
            logger(0, 'results: %d' % len(results['items']))
        except:
            logger(0, 'Fail')
            results = {'items':[]}
        for item in results['items']:
            self.parseStory(feedobj, item)

    security.declareProtected(p_manageRSSCollection , 'aggregate')
    def aggregate(self, REQUEST=None):
        """ Now, let it runs with the feeds """
        feeds = self.objectValues('RSSFeed')
        for feed in feeds:
            if feed.status==STATUS_DEAD: 
                logger(0, 'Bypass %s because it was dead!' %feed.getId())
                continue #bypass dead feed if u do not want, please reset it's status
            self.parseFeed(feed)
        logger(0, 'Aggregation Complete')

        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect( self.absolute_url()
                                        + '/rsscollection_view'
                                        + '?portal_status_message=Collection+updated'
                                        )

    security.declareProtected(p_manageRSSCollection , 'importFeeds')
    def importFeeds(self, feeds):
        """  """
        for feed in feeds:
            title = '%(sitename)s (%(siteurl)s, %(dataurl)s)' %feed
            dataurl = feed['dataurl']
            description = feed['description']
            fetchable = feed['fetchable']
            status = feed['status']
            categories = feed['Categories']

            id = b2a_base64(md5(dataurl).digest()).strip()
            id = id.translate(_base64_to_id_trans, _delete_chars)

            while id and (id[0] == '_' or id[0] == '-'):
                id=id[1:]
            while id and (id[-1] == '_' or id[-1] == '-'):
                id=id[:-1]

            if not hasattr(self.aq_explicit, id):
                self.invokeFactory( type_name='RSSFeed', 
                                    id =id, 
                                    title=title, 
                                    description=description, 
                                    dataurl=dataurl, 
                                    fetchable=fetchable, 
                                    status=status, 
                                    categories=categories)

InitializeClass(RSSCollection)
